class Part_of_Speech_Tagger:
    """
    The Part of Speech (POS) Tagger is responsible for identifying the correct part of speech for each token
    of any given Arabic sentence.
    The POS Tagger relies heavily on the Morphological Analyzer to extract the relevant morpho-syntactic
    features for the input words.
    The POS Tagger also relies on the Auto-Corrector to correct input text.
    """

    def __init__(self, app_id):
        import zeep
        self.__WSDL = "https://atks.microsoft.com/Services/POSTaggerService.svc"
        self.__PORT = "HTTPS_IPOSTaggerService"
        self.__client = zeep.Client(wsdl=self.__WSDL, port_name=self.__PORT)
        self.__app_id = app_id

    def ReportWrongTag(self, word, context):
        """
        :param word: string
        :param context: string
        :return: ns1:POSTaggerErrorCode
        """
        result = self.__client.service.ReportWrongTag(self.__app_id, word, context)
        return result
    def SuggestTag(self, word, context, suggested_tag):
        """
        :param word: string
        :param context: string
        :param suggested_tag: string
        :return: ns1:POSTaggerErrorCode
        """
        result = self.__client.service.SuggestTag(self.__app_id, word, context, suggested_tag)
        return result
    def TagText(self, text, auto_correct):
        """
        :param text: string
        :param auto_correct: boolen
        :return: ns1:POSTaggerErrorCode, taggedWords: ns1:ArrayOfTaggedWord
        """
        result = self.__client.service.TagText(self.__app_id, text, auto_correct)
        return result

    def TagWords(self, words):
        """
        :param words: list of strings
        :return: ns1:POSTaggerErrorCode, taggedWords: ns1:ArrayOfString
        """
        result = self.__client.service.TagWords(self.__app_id, words)
        return result



class Parser:
    """
    The Arabic Parser determines the grammatical structure of Arabic sentences,
    such as which groups of words combine to form phrases and which words are the subject or the object of a verb.
    The Parser relies heavily on the Arabic POS Tagger to identify the correct part of speech for each token
    in an input Arabic sentence, and the Arabic Named-Entity Recognizer to identify named entities in the input
    sentence after it has been corrected using the Arabic Auto-Corrector.
    """

    def __init__(self, app_id):
        import zeep
        self.__WSDL = "https://atks.microsoft.com/Services/ParserService.svc"
        self.__PORT = "HTTPS_IParserService"
        self.__client = zeep.Client(wsdl=self.__WSDL, port_name=self.__PORT)
        self.__app_id = app_id

    def Parse(self, in_text):
        """

        :param in_text: string
        :return: ns1:ParserErrorCode, parseTree: xsd:string, score: xsd:double
        """
        result = self.__client.service.Parse(self.__app_id, in_text)
        return result

    def SuggestParseTree(self, in_text, alternative_parse_tree):
        """
        :param in_text: string
        :param alternative_parse_tree: string
        :return: ns1:ParserErrorCode
        """
        result = self.__client.service.SuggestParseTree(self.__app_id, in_text, alternative_parse_tree)
        return result


class Transliteration:
    """
    Transliteration is the conversion of text from one script to another while preserving
    the same pronunciation. The Transliterator provides translation of named entities,
    such as human and city names, from English to Arabic and vice versa—and conversion of text
    from Romanized Arabic to native Arabic script.
    """

    def __init__(self, app_id):
        import zeep
        self.__WSDL = "https://atks.microsoft.com/Services/TransliteratorService.svc"
        self.__PORT = "HTTPS_ITransliteratorService"
        self.__client = zeep.Client(wsdl=self.__WSDL, port_name=self.__PORT)
        self.__app_id = app_id

    def IsRomanizedArabic(self, text):
        """
        :param text: string
        :return: ns1:TransliteratorErrorCode, isRomanizedArabic: xsd:boolean, confidence: xsd:double
        """
        result = self.__client.service.IsRomanizedArabic(self.__app_id, text)
        return result

    def ReportCandidateSelection(self, word, context, transliteration):
        """
        :param word: string
        :param context: string
        :param transliteration: string
        :return: ns1:TransliteratorErrorCode
        """
        result = self.__client.service.ReportCandidateSelection(self.__app_id, word, context, transliteration)
        return result

    def SuggestMissingTransliteration(self, word, context, transliteration):
        """
        :param word: string
        :param context: string
        :param transliteration: string
        :return: ns1:TransliteratorErrorCode
        """
        result = self.__client.service.SuggestMissingTransliteration(self.__app_id, word, context, transliteration)
        return result

    def TransliterateText(self, in_text, lang_pair):
        """
        :param in_text: string
        :param lang_pair: LanguagePair
        :return: ns1:TransliteratorErrorCode, outText: xsd:string
        """
        result = self.__client.service.TransliterateText(self.__app_id, in_text, lang_pair)
        return result

    def TransliterateWords(self, tokens, lang_pair):
        """
        :param tokens: list of strings
        :param lang_pair: LanguagePair
        :return: ns1:TransliteratorErrorCode, outTokens: ns1:ArrayOfString
        """
        result = self.__client.service.TransliterateWords(self.__app_id, tokens, lang_pair)
        return result



class Colloquial_Converter:
    """The Colloquial Converter provides translation of Egyptian colloquial text
     into the equivalent Modern Standard Arabic text along with rich mapping information.
      Both Sarf and the Colloquial Morphological Analyzer are used internally to provide the
      final translation."""

    def __init__(self, app_id):
        import zeep
        self.__WSDL = "https://atks.microsoft.com/Services/ColloquialConverterService.svc"
        self.__PORT = "HTTPS_IColloquialConverterService"
        self.__client = zeep.Client(wsdl=self.__WSDL, port_name=self.__PORT)
        self.__app_id = app_id

    def ConvertText(self, in_text):
        """
        :param in_text: string
        :return: ns1:ColloquialConverterErrorCode, outText: xsd:string

        """
        result = self.__client.service.ConvertText(self.__app_id, in_text)
        return result

    def ConvertTextWithDetails(self, in_text):
        """
        :param in_text: string
        :return: ns1:ColloquialConverterErrorCode, outText: xsd:string, words: ns1:ArrayOfColloquialWordMap

        """
        result = self.__client.service.ConvertTextWithDetails(self.__app_id, in_text)
        return result

    def ConvertWord(self, word):
        """
        :param word: string
        :return: ns1:ColloquialConverterErrorCode, translations: ns1:ArrayOfString
        """
        result = self.__client.service.ConvertWord(self.__app_id, word)
        return result

    def SuggestConversion(self, in_text, suggested_text):
        """
        :param in_text: string
        :param suggested_text: string
        :return: ns1:ColloquialConverterErrorCode
        """
        result = self.__client.service.SuggestConversion(self.__app_id, in_text, suggested_text)
        return result



from .Colloquial_Converter import Colloquial_Converter
from .Diacritizer import Diacritizer
from .Named_Entity_Recognizer import Named_Entity_Recognizer
from .Parser import Parser
from .Part_of_Speech_Tagger import Part_of_Speech_Tagger
from .SARF import SARF
from .Speller import Speller
from .Transliteration import Transliteration


class Speller:
    """
    The Speller detects and corrects misspelled words in Arabic text and is designed for
    Modern Standard Arabic. The Speller APIs also enable auto-correction of Common Arabic Mistakes,
    frequent orthographical errors. The main objective of the Speller is to enhance the quality
    of written Arabic text, hence improving the accuracy of the various Arabic text-processing components.
    """

    def __init__(self, app_id):
        import zeep
        self.__WSDL = "https://atks.microsoft.com/Services/SpellerService.svc"
        self.__PORT = "HTTPS_ISpellerService"
        self.__client = zeep.Client(wsdl=self.__WSDL, port_name=self.__PORT)
        self.__app_id = app_id

    def Autocorrect(self, in_text, process_last_word_only):
        """
        :param in_text: string
        :param process_last_word_only: boolean
        :return: ns1:SpellerErrorCode, correctedText: xsd:string
        """
        result = self.__client.service.Autocorrect(self.__app_id, in_text, process_last_word_only)
        return result

    def AutocorrectAndSuggest(self, in_text, process_last_word_only):
        """
        :param in_text: string
        :param process_last_word_only: boolean
        :return: ns1:SpellerErrorCode, correctedText: xsd:string, erroneousWords: ns1:ArrayOfErroneousWord
        """
        result = self.__client.service.AutocorrectAndSuggest(self.__app_id, in_text, process_last_word_only)
        return result

    def AutocorrectAndSuggest2(self, in_text, correction_options, process_last_word_only):
        """
        :param in_text: string
        :param correction_options: SuggestionOption
        :param process_last_word_only: boolean
        :return: ns1:SpellerErrorCode, correctedText: xsd:string, erroneousWords: ns1:ArrayOfErroneousWord
        """
        result = self.__client.service.AutocorrectAndSuggest2(self.__app_id, in_text, correction_options,
                                                              process_last_word_only)
        return result

    def DetectMistakes(self, in_text, process_last_word_only):
        """
        :param in_text: string
        :param process_last_word_only: boolean
        :return: ns1:SpellerErrorCode, erroneousWords: ns1:ArrayOfErroneousWord
        """
        result = self.__client.service.DetectMistakes(self.__app_id, in_text, process_last_word_only)
        return result

    def ReportCorrectionSelection(self, word, context, correction):
        """
        :param word: string
        :param context: string
        :param correction: string
        :return: ns1:SpellerErrorCode
        """
        result = self.__client.service.ReportCorrectionSelection(self.__app_id, word, context, correction)
        return result

    def ReportWronglyDetectedWord(self, word, context):
        """
        :param word: string
        :param context: string
        :return: ns1:SpellerErrorCode
        """
        result = self.__client.service.ReportWronglyDetectedWord(self.__app_id, word, context)
        return result

    def SuggestCorrectionForSpecificWord(self, in_text, word_position, correction_options):
        """
        :param in_text: string
        :param word_position: int
        :param correction_options: SuggestionOption
        :return: ns1:SpellerErrorCode, erroneousWord: ns1:ErroneousWord
        """
        result = self.__client.service.SuggestCorrectionForSpecificWord(self.__app_id, in_text, word_position,
                                                                        correction_options)
        return result

    def SuggestCorrections(self, in_text, correction_options, process_last_word_only):
        """
        :param in_text: string
        :param correction_options: SuggestionOption
        :param process_last_word_only: boolean
        :return: ns1:SpellerErrorCode, erroneousWords: ns1:ArrayOfErroneousWord
        """
        result = self.__client.service.SuggestCorrections(self.__app_id, in_text, correction_options,
                                                          process_last_word_only)
        return result

    def SuggestMissingCorrection(self, word, context, correction):
        """
        :param word: string
        :param context: string
        :param correction: string
        :return: ns1:SpellerErrorCode
        """
        result = self.__client.service.SuggestMissingCorrection(self.__app_id, word, context, correction)
        return result


from setuptools import setup

setup(name='atkspy',
      version='0.1',
      description='A python package that supports SOAP interface to communicate with the Microsoft ATKS',
      url='https://github.com/AliAbdelaal/ATKSpy',
      author='Ali Abdelaal',
      author_email='aliabdelaal369@gmail.com',
      license='MIT',
      packages=['atkspy'],
      install_requires=[
          'zeep',
      ],
      python_requires='>=3',
      zip_safe=False)


class Named_Entity_Recognizer:
    """
    The Arabic Named Entity Recognizer (ANER) detects and classifies named entities in Arabic text.
    It classifies them into three categories: persons, locations, and organizations.
    It also provides a character index at which the named entity is located in the original text.

    ANER is adopting hybrid deterministic/probabilistic approaches for detecting and classifying named entities.
    The user can use the appropriate ANER setup per call, but using all the components provides the best quality.
    Processing article by article instead of sentence by sentence provides more contextual information,
    which contributes to higher detection and classification accuracy.

    ANER uses the Speller component to enhance the quality of the Arabic text before the named-entity-detection phase.
    The detection/classification accuracy depends on the quality of the input Arabic text.
    The user has the option to select between them depending on the error rate of the
    input text: Auto-Corrector for low error rate and Speller for high error rate.
    """

    def __init__(self, app_id):
        import zeep
        self.__WSDL = "https://atks.microsoft.com/Services/ANERService.svc"
        self.__PORT = "HTTPS_IANERService"
        self.__client = zeep.Client(wsdl=self.__WSDL, port_name=self.__PORT)
        self.__app_id = app_id

    # TODO :: build the options part
    def GetArabicNamedEntities(self, arabic_text, options=None):
        """
        :param arabic_text: string
        :param options: list of strings can take values of the following:
        'UseAllComponents' , 'UseAutoCorrector', 'UseSpeller'
        example : options = ['UseAllComponents']
        :return: ns1:NERErrorCode, namedEntities: ns1:ArrayOfNamedEntity
        """
        if options is None:
            options = ['UseAllComponents']
        result = self.__client.service.GetArabicNamedEntities(self.__app_id, arabic_text, options)
        return result

    def ReportWronglyDetectedNamedEntity(self, named_entity, context, wrong_named_entity):
        """
        :param named_entity: string
        :param context: string
        :param wrong_named_entity: string
        :return: ns1:NERErrorCode
        """

    def SuggestMissingNamedEntity(self, named_entity, context, named_entity_type):
        """
        :param named_entity: string
        :param context: string
        :param named_entity_type: string
        :return: ns1:NERErrorCode
        """
        result = self.__client.service.SuggestMissingNamedEntity(self.__app_id, named_entity, context, named_entity_type)
        return result



class Diacritizer:
    """ The Arabic Automatic Diacritizer component performs vowel restoration on input Arabic text.
     The main objective of the Diacritizer is to insert both missing vowels—diacritics—of the stem
     and the missing vowel for the case ending. """

    def __init__(self, app_id):
        import zeep
        self.__WSDL = "https://atks.microsoft.com/Services/DiacritizerService.svc"
        self.__PORT = "HTTPS_IArabicDiacritizerService"
        self.__client = zeep.Client(wsdl=self.__WSDL, port_name=self.__PORT)
        self.__app_id = app_id

    def Diacritize(self, text, enable_case_ending, enable_quran):
        """
        :param text: string
        :param enable_case_ending: boolean
        :param enable_quran: boolean
        :return: ns1:DiacritizerErrorCode, diacritizedText: xsd:string
        """
        result = self.__client.service.Diacritize(self.__app_id, text, enable_case_ending, enable_quran)
        return result

    def DiacritizeWithTraceInfo(self, text, enable_case_ending, enable_quran, enable_classics):
        """
        :param text: string
        :param enable_case_ending: boolean
        :param enable_quran: boolean
        :param enable_classics: boolean
        :return: ns1:DiacritizerErrorCode, diacritizedText: xsd:string, traceInfo: ns1:ArrayOfWordTraceInfo
        """
        result = self.__client.service.DiacritizeWithTraceInfo(self.__app_id, text, enable_case_ending, enable_quran,
                                                               enable_classics)
        return result

    def ReportWrongDiacritization(self, word, context, suggested_diacritized_word):
        """
        :param word: string
        :param context: string
        :param suggested_diacritized_word: string
        :return: ns1:DiacritizerErrorCode
        """
        result = self.__client.service.ReportWrongDiacritization(self.__app_id, word, context, suggested_diacritized_word)
        return result

    def SuggestDiacritization(self, word, context, suggested_diacritized_word):
        """
        :param word: string
        :param context: string
        :param suggested_diacritized_word: string
        :return: ns1:DiacritizerErrorCode
        """
        result = self.__client.service.SuggestDiacritization(self.__app_id, word, context, suggested_diacritized_word)
        return result



class SARF:
    """
    Sarf provides automatic morphological analysis of Arabic words.
    It provides all possible morphological analyses for any given input Arabic word.
    Each analysis consists of the diacritized word and the morphological breakdown of
    the analysis in terms of prefixes, stem, and suffixes.
    The stem is further decomposed into its root and morphological pattern.
    Moreover, each analysis carries the part of speech and a set of morpho-syntactic
    features such as gender, number, transitivity, verb voice, and verb mood.
    """

    def __init__(self, app_id):
        import zeep
        self.__WSDL = "https://atks.microsoft.com/Services/SarfService.svc"
        self.__PORT = "HTTPS_ISarfService"
        self.__client = zeep.Client(wsdl=self.__WSDL, port_name=self.__PORT)
        self.__app_id = app_id

    def AnalyzeToken(self, word, ling_mode):
        """
        :param word: string
        :param ling_mode: LinguisticMode
        ling_mode can take values
            Classical	    0	Default operational mode (ignore direct speech analyses)
            Colloquial	    1	Not used
            Conversational	2	Keeps direct speech analyses (use this to generate all analyses including direct speech analyses)
        :return: ns1:SarfErrorCode, analyses: ns1:ArrayOfSarfAnalysis
        """
        result = self.__client.service.AnalyzeToken(self.__app_id, word, ling_mode)
        return result

    def AnalyzeTokensArray(self, words, ling_mode):
        """
        :param words: array of strings
        :param ling_mode: LinguisticMode
        ling_mode can take values
            Classical	    0	Default operational mode (ignore direct speech analyses)
            Colloquial	    1	Not used
            Conversational	2	Keeps direct speech analyses (use this to generate all analyses including direct speech analyses)
        :return: ns1:ArrayOfSarfErrorCode, analyses: ns1:ArrayOfArrayOfSarfAnalysis
        """
        result = self.__client.service.AnalyzeTokensArray(self.__app_id, words, ling_mode)
        return result

    def DecodeFeatures(self, binary_syntactic_features):
        """
        :param binary_syntactic_features: long
        :return: ns1:SarfErrorCode, detailedFeatures: ns1:MorphoSyntacticFeatures
        """
        result = self.__client.service.DecodeFeatures(self.__app_id, binary_syntactic_features)
        return result

    def EncodeFeatures(self, morpho_syntactic_features):
        """
        :param morpho_syntactic_features: long
        :return: ns1:SarfErrorCode, binarySyntacticFeatures: xsd:long
        """
        result = self.__client.service.EncodeFeatures(self.__app_id, morpho_syntactic_features)
        return result

    def GetAllDerivatives(self, analysis, ignore_affixation):
        """
        :param analysis: sarfAnalysis
        :param ignore_affixation: boolean
        :return: ns1:SarfErrorCode, analyses: ns1:ArrayOfSarfAnalysis
        """
        result = self.__client.service.GetAllDerivatives(self.__app_id, analysis, ignore_affixation)
        return result

    def GetDerivatives(self, analysis, pos, ignore_affixation):
        """
        :param analysis: SarfAnalysis
        :param pos: PartOfSpeech
        :param ignore_affixation: boolean
        :return: ns1:SarfErrorCode, analyses: ns1:ArrayOfSarfAnalysis
        """
        result = self.__client.service.GetDerivatives(self.__app_id, analysis, pos, ignore_affixation)
        return result

    def GetInflections(self, analysis, ling_mode, diacritized):
        """
        :param analysis: SarfAnalysis
        :param ling_mode: LinguisticMode
        ling_mode can take values
            'Classical'	    Default operational mode (ignore direct speech analyses)
            'Conversational'	Keeps direct speech analyses (use this to generate all analyses including direct speech analyses)
        :param diacritized: boolean
        :return: ns1:SarfErrorCode, inflections: ns1:ArrayOfString, totalInflections: xsd:int
        """
        results = self.__client.service.GetInflections(self.__app_id, analysis, ling_mode, diacritized)
        return results

    def GetInflectionsEx(self, analysis, ling_mode, diacritized, max_inflections, start_index):
        """
        :param analysis: SarfAnalysis
        :param ling_mode: LinguisticMode
        ling_mode can take values
            'Classical'	    Default operational mode (ignore direct speech analyses)
            'Conversational'	Keeps direct speech analyses (use this to generate all analyses including direct speech analyses)
        :param diacritized: boolean
        :param max_inflections: int
        :param start_index: int
        :return: ns1:SarfErrorCode, inflections: ns1:ArrayOfString, totalInflections: xsd:int
        """
        result = self.__client.service.GetInflectionsEx(self.__app_id, analysis, ling_mode, diacritized,
                                                        max_inflections, start_index)
        return result

    def GetPlural(self, analysis, ignore_affixation):
        """
        :param analysis: SarfAnalysis
        :param ignore_affixation: boolean
        :return: ns1:SarfErrorCode, analyses: ns1:ArrayOfSarfAnalysis
        """
        result = self.__client.service.GetPlural(self.__app_id, analysis, ignore_affixation)
        return result